%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% A. INPUT DATA %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
%%%% Cell Filling Parameters %%%%
NmodesOptim=2562;                                     %Number of optimization variables for the filling algorithm (optimized for n=2 to n=NvarOptim+1)

%%%% Fourier Spectrum Properties %%%%
TypeSpectrum=0;                                     %0:Taylored spectrum ; 1:Existing spectrum

%%%% If TypeSpectrum is 0 %%%%
DescriptorD2=0.05;                                   %Fourier descriptor n=2
DescriptorD3=0.05;                                   %Fourier descriptor n=3
SpectrumDecay1=-1.6;                                  %Exponential decay from n=3 to n=7
DescriptorD8=0.002;                                 %Fourier descriptor n=8
SpectrumDecay2=-1.6;                                  %Exponential decay from n>8

%%%% If TypeSpectrum is 1 %%%%
FileSpectrum='Spectrum_ExampleSand.mat';            %Name of a Spectrum file

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% B. COMPUTATIONS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%     PRESS F5    %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TargetSolidFraction=1; 
geode='geode4.mat';
load(geode)
load('CalibrationMesh.mat')
Nsample=size(Cellules,1);
if TypeSpectrum==0
    if DescriptorD2==0
        DescriptorD2=10^-5;
    end
    if DescriptorD3==0
        DescriptorD3=10^-5;
    end
    if DescriptorD8==0
        DescriptorD8=10^-5;
    end
    [Cn]=Spectre(128,1,0,DescriptorD2,DescriptorD3,SpectrumDecay1,DescriptorD8,SpectrumDecay2);
else
    load(FileSpectrum)
end
disp(' ')
disp(['Construction of the autocorrelation matrix'])
disp(' ')
Autocov_Mat=Mat_Construction(Cn,geode);
[EigenVectors,EigenValues]=eig(Autocov_Mat);
EigenVectors=flipud(sortrows(cat(2,real(diag(EigenValues)),real(EigenVectors)'),1));
EigenValues=EigenVectors(:,1);
EigenVectors=EigenVectors(:,2:size(EigenVectors,2))';

EigensCells=[];
for i=1:size(Cellules,1)
    disp(['Pre-filling Particle ',num2str(i),'/',num2str(size(Cellules,1))])
    RadCell=RadsCells(1:size(RadsCells,1),i);
    RadCellNorm=RadCell/mean(RadCell);
    EigenCell=zeros(size(EigenVectors,2),1);
    for j=1:size(EigenVectors,2)
        EigenCell(j,1)=RadCellNorm'*EigenVectors(:,j);
    end
    EigensCells(1:size(EigenCell,1),i)=EigenCell;
end
disp(' ')

Randoms=zeros(size(EigensCells,1),size(EigensCells,2));
for i=1:size(EigensCells,2)
    Randoms(:,i)=EigensCells(:,i)./(EigenValues(:,1).^0.5);
end
Correct=cat(2,mean(Randoms')',std(Randoms')');

for i=1:size(Cellules,1)
    disp(['Filling Particle ',num2str(i),'/',num2str(size(Cellules,1))])
    %Points=Vertices(Cellules{i,1},:);
    %Tri=convhulln(Points);
    %[Points_Cell,Struc_Cell]=Tri2Domain(Points,Tri);
    Centre=Centres(i,:);
    EigenCell=EigensCells(:,i);
    RadCell=RadsCells(:,i);
    [Xgrain,Ygrain,Zgrain,Rgrain]=FillCell3D(Centre,EigenVectors,EigenValues,EigenCell,RadCell,NmodesOptim,geode,Correct,VolumesCells(i,1),TargetSolidFraction);
    VolumeGrain=Volume_Geode(Rgrain,Structure,Angles);
    Grains{i,1}=cat(2,Xgrain,Ygrain,Zgrain,Rgrain);
    VolumesGrains(i,1)=VolumeGrain;
end
disp(' ')

Properties=zeros(size(Grains,1),19);
ListSpheres=cell(size(Grains,1),1);
for i=1:size(Grains,1)
    disp(['Analysing Particle ',num2str(i),'/',num2str(size(Cellules,1))])
    Rmax=max(Grains{i,1}(:,4));
    [Pd,Sd]=Tri2Domain(Grains{i,1}(:,1:3),Structure);
    [Spheres,PointsOK]=ODEC_Filling(Pd,Sd,Rmax/100,Rmax/100,0.001);
    ListSpheres{i,1}=Spheres;
    [Flatness,Elongation,AspectRatio,Roundness,Circularity,Regularity,Tx,Ty,Tz,S,I,L,Rcirc,Rinsc,Surf,Surf_Conv,Vol,Vol_Conv,Q,Q_Conv]=Proprietes(Grains{i,1}(:,1),Grains{i,1}(:,2),Grains{i,1}(:,3),Grains{i,1}(:,4),geode,Spheres);
    Properties(i,1)=Flatness;
    Properties(i,2)=Elongation;
    Properties(i,3)=AspectRatio;
    Properties(i,4)=Roundness;
    Properties(i,5)=Circularity;
    Properties(i,6)=Regularity;
    Properties(i,7)=Tx;
    Properties(i,8)=Ty;
    Properties(i,9)=Tz;
    Properties(i,10)=S;
    Properties(i,11)=I;
    Properties(i,12)=L;
    Properties(i,13)=Rcirc;
    Properties(i,14)=Rinsc;
    Properties(i,15)=Surf;
    Properties(i,16)=Surf_Conv;
    Properties(i,17)=Vol;
    Properties(i,18)=Vol_Conv;
    Properties(i,19)=Q;
    Properties(i,20)=Q_Conv;
end
Plot_Sample(geode,'Unit_Cube_Domain.mat',Cellules,Vertices,Centres,Grains,VolumesGrains,'Calibration')
disp(' ')
disp(['Maximum Solid Fraction : ',num2str(sum(VolumesGrains))])
disp(['Average Flatness : ',num2str(mean(Properties(:,1)))])
disp(['Average Elongation : ',num2str(mean(Properties(:,2)))])
disp(['Average Aspect ratio : ',num2str(mean(Properties(:,3)))])
disp(['Average Roundness : ',num2str(mean(Properties(:,4)))])
disp(['Average Sphericity : ',num2str(mean(Properties(:,5)))])
disp(['Average Isoperimetric quotient : ',num2str(mean(Properties(:,19)))])