%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                                       %%
%%                      Packing 3D                       %%
%%                                                       %%
%%                     Main Program                      %%
%%              Version 1.1 ; January 2014               %%
%%                                                       %%
%%                Author: Guilhem Mollon                 %%
%%               Supervisor: Jidong Zhao                 %%
%%                                                       %%
%%       Realized at Universit Joseph Fourier,          %%
%%                  Grenoble, France                     %%
%%                     Year 2013                         %%
%%                                                       %%
%%            Please read enclosed .txt file             %%
%%                                                       %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%% A. INPUT DATA %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%% Created data file %%%%
clear all
FileName='PubliFacet.mat';                     %Name of the created MATLAB data file for the sample
MeshName='PubliFacetMesh.mat';                            %Name of the created MATLAB data file for the constrained mesh

%%%% Constrained Voronoi Tessellation parameters %%%%
Nparticles=1000;                                    %Number of generated points in the domain
TargetVolumeCOV=0.1;                                %Target COV of the grains surfaces
Domain='Unit_Hopper_Domain.mat';                  %Filename of the domain to be filled with particles
NiterMax=100000;                                    %Maximum number of iterations for Constrained Voronoi Tessellation
TargetError=0.1;                                    %Target error for Constrained Voronoi Tessellation

%%%% Cell Filling Parameters %%%%
TargetSolidFraction=0.2;                           %Target solid fraction
NmodesOptim=2562;                                     %Number of optimization variables for the filling algorithm (optimized for n=2 to n=NvarOptim+1)

%%%% Fourier Spectrum Properties %%%%
TypeSpectrum=0;                                     %0:Taylored spectrum ; 1:Existing spectrum

%%%% If TypeSpectrum is 0 %%%%
DescriptorD2=0.05;                                 %Fourier descriptor n=2
DescriptorD3=0.04;                                  %Fourier descriptor n=3
SpectrumDecay1=-1.6;                                  %Exponential decay from n=3 to n=7
DescriptorD8=0.007;                                %Fourier descriptor n=8
SpectrumDecay2=-1.6;                                  %Exponential decay from n>8

%%%% If TypeSpectrum is 1 %%%%
FileSpectrum='Spectrum_ExampleSand.mat';            %Name of a Spectrum file

%%%% ODECS Algorithm Parameters %%%%
Dmax=0.02;                                          %Maximum distance for which a contour point is considered as "covered" by a given disc
Rmin=0.02;                                          %Minimum radius of a given disc
Pmax=0.05;                                          %Accepted proportion of "uncovered" contour points

%%%% Sample statistics %%%%
Nstat=1000;                                         %Number of particles randomly analysed for shape statistics

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%% B. COMPUTATIONS %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%     PRESS F5    %%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[X,Y,Z,Neighbors,ConvergenceHistory,VoronoiCells,VoronoiVertices,Pdf_Volumes,Pdf_Target,Grains,Centres,VolumesGrains,VolumesCells,ODECS,SampleProperties,FourierSpectrum,Inertias]=Secondary_Program(FileName,MeshName,Nparticles,TargetVolumeCOV,Domain,NiterMax,TargetError,TargetSolidFraction,NmodesOptim,TypeSpectrum,FileSpectrum,DescriptorD2,DescriptorD3,SpectrumDecay1,DescriptorD8,SpectrumDecay2,Dmax,Rmin,Pmax,Nstat);
save(FileName)
Plot_Sample('Geode4.mat',Domain,VoronoiCells,VoronoiVertices,Centres,Grains,VolumesGrains,FileName)